/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.mike;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.content.pm.PackageManager;
import android.media.AudioAttributes;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Build;
import android.os.Bundle;
import android.os.Process;
import android.view.WindowManager;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;

import static android.os.Process.setThreadPriority;

public class MainActivity extends AppCompatActivity {

    // Static vars
    private static final int RC_PERMISSIONS = 1000;

    // Views
    private TextView textState;
    private Switch switchOnOff, switchMute, switchStandby;

    // State vars
    private boolean recording = false;
    private boolean muted = false;
    private boolean standby = false;

    // Sound vars
    private MediaPlayer soundStandby;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Set flags so Activity is displayed above lock screen
        getWindow().addFlags(
                WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON |
                WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD |
                WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED |
                WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
        );
        setContentView(R.layout.activity_main);

        // Init views
        textState = findViewById(R.id.main_text_state);
        switchOnOff = findViewById(R.id.main_switch_on_off);
        switchMute = findViewById(R.id.main_switch_mute);
        switchStandby = findViewById(R.id.main_switch_standby);

        // Set view props
        switchOnOff.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    recordStart();
                }else{
                    // When switch off
                    recordStop();
                }
            }
        });
        switchMute.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    muted = true;
                    updateUI();
                }else{
                    // When switch off
                    muted = false;
                    updateUI();
                }
            }
        });
        switchStandby.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(b){
                    // When switch on
                    standby = true;

                    // Start sound
                    soundStandby.seekTo(0);
                    soundStandby.start();

                    // Update UI
                    updateUI();
                }else{
                    // When switch off
                    standby = false;

                    // Stop sound
                    soundStandby.pause();

                    // Update UI
                    updateUI();
                }
            }
        });

        // Load sounds
        soundStandby = MediaPlayer.create(this, R.raw.sound_standby);
        soundStandby.setLooping(true);
        soundStandby.setVolume(0.05f, 0.05f);
    }


    private void recordStart(){
        if(recording) return; // Return if already recording

        // Check if permission to use microphone is granted
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M &&
                ContextCompat.checkSelfPermission(this, Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_DENIED){
            // Show dialog to user
            requestPermissions(new String[]{Manifest.permission.RECORD_AUDIO}, RC_PERMISSIONS);
            return;
        }

        // Start recording
        recording = true;

        // Start thread
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {

                // Init recorder
                int minsize = AudioRecord.getMinBufferSize(
                        44100,
                        AudioFormat.CHANNEL_IN_MONO,
                        AudioFormat.ENCODING_PCM_16BIT
                );
                AudioRecord audioRecord = new AudioRecord(
                        MediaRecorder.AudioSource.MIC,
                        44100,
                        AudioFormat.CHANNEL_IN_MONO,
                        AudioFormat.ENCODING_PCM_16BIT,
                        minsize
                );

                // Init player
                setThreadPriority(Process.THREAD_PRIORITY_URGENT_AUDIO);
                AudioTrack audioTrack = new AudioTrack(
                        AudioManager.STREAM_MUSIC,
                        44100,
                        AudioFormat.CHANNEL_OUT_MONO,
                        AudioFormat.ENCODING_PCM_16BIT,
                        minsize,
                        AudioTrack.MODE_STREAM
                );

                // Start
                audioRecord.startRecording();
                audioTrack.play();

                // Loop
                while(recording){

                    // Get new data
                    short[] buffer = new short[minsize];
                    audioRecord.read(buffer, 0, minsize);

                    // Manipulate data

                    // Mute
                    if(muted || standby){
                        // Empty buffer
                        buffer = new short[minsize];

                        for(int i=0; i<minsize; i++){
                            buffer[i] = 0;
                        }
                    }

                    // Write data to output
                    audioTrack.write(buffer, 0, minsize);
                }

                // Stop recording
                audioRecord.stop();
                audioTrack.stop();

            }
        });
        thread.start();

        // Change UI
        updateUI();
    }

    private void recordStop(){
        if(!recording) return; // Return if not recording

        // Stop recording
        recording = false;

        // Change UI
        updateUI();
    }


    private void updateUI(){

        // Set right state text
        String stateText = "IDLE";
        if(recording){
            stateText = "ON AIR";

            if(muted) stateText = "MUTED";
            if(standby) stateText = "STAND BY";
        }

        textState.setText(stateText);

    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        // Check request code
        switch(requestCode){
            case RC_PERMISSIONS:
                // Check if granted
                if(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED){
                    // Start recording
                    recordStart();
                }else{
                    // Set main switch to off
                    switchOnOff.setChecked(false);
                }
                return;
        }
    }
}